function  [routing_dist, routing_load, storage_load, dht_packets, cluster, dv_packets, dv_entries] = ...
    compactrouting(neighbor_dist,pre_landmarks, ...
		   test_pairs, failure_nodes, reactive,coord, study_dht)
%Refer to "Compact routing schemes" by Thorup & Zwick for the
%algorithm 
%neighbor_dist: distances are arbitrary ones, not necessarily
%hops. inf indicates non-connectivity.
% 
% pre_landmarks: pre-set landmark sets. 
% if not given, used the proposed randomized algorithm in the paper
% if length(pre_landmarks)==1, it's used as the number of landmarks
% note that in study_cr(), landmarks are always pre-specified.

% test_pairs: the src-dest pairs selected for routing performance evaluation
% failure_nodes: the nodes that may fail in routing test, but not
% in routing table establishment stage.

% reactive: 0: local flooding; 1: reactive; 2: Distance Vector

% coord: the physical coordinate for debug visualization only
% return values:
% routing_dist(): the routing distance for each test pair. Inf
% means unrechable
% routing_load: the load on every node
% storage_load: the storage load on every beacon
% dht_packet: how many packets does it take to setup the DHT
% cluster: the info about the cluster (routing table) size

debug = 1;
debug_route = 0; %this will generate a lot of debug information
if ~issparse(neighbor_dist)
  neighbor_dist = dist_full2sparse(neighbor_dist);
end
if nargin<5
  reactive = 0; %default CR is proactive, i.e. reactive = 0;
end
if nargin<7
  study_dht = 0;
end

n = length(neighbor_dist);
%fprintf('calculating best path...');
%[delta optimal_nexthop] = floyd_warshall(neighbor_dist);
%[delta optimal_nexthop] = allpair_dijkstra(neighbor_dist);
%fprintf('done\n');

%%%select landmarks
W = 1:n;
if nargin<2
  pre_landmarks = []; 
end

if nargin<3
  tmp1 = repmat(1:n,n,1);
  tmp2 = tmp1';
  rest_pairs = [tmp1(:) tmp2(:)];
end

if nargin<4
  failure_nodes = [];
end
tmp = failure_nodes;
failure_nodes = zeros(n,1);
failure_nodes(tmp) = ones(length(tmp),1);

fprintf(['*CR: running mode:%d (0:scoped flooding; 1:' ...
	 ' reactive; 2: scoped DV)\n'], reactive);

out_degree = full(sum(neighbor_dist>0,2));
%C = zeros(n,n); %C(w,v)==1 means v \in C(w)
%routing_table(w,v)=nexthop, if v \in C(w)
routing_table = sparse([]);
routing_table(n,n) = 0;
%direct_dist = routing_table; %should be sync with routing_table:
                             %direct_dist(w,v)=shortest_dist bwteen w&v
for i=1:n
  tmp = find(neighbor_dist(:,i));
  routing_table(tmp,i)=repmat(i,length(tmp),1);
end
%routing_table
direct_dist = neighbor_dist;
%nexthop2bcn(i,b) = nexthop(i,b) where b is the relative beacon id
deltaA = zeros(1,n); % deltaA(v) == \delta(A,v)
%cent(u), the closest landmark to each node, relative index to A

A = [];
s = sqrt(n*log(n)/log(2));
%s = sqrt(n);%!!!!
while length(W)>0
  if length(pre_landmarks)>=1%!!!!
    A = pre_landmarks;
  elseif length(pre_landmarks)==1
    % pre_landmarks is defined as the # of random landmarks
    tmp = randperm(n);
    A = tmp(1:pre_landmarks);
  else
    A = union(A, sample(W,s));
  end
  A = sort(A);
  ordinary_nodes = setdiff(1:n, A);
  isbeacon = zeros(n,1);
  isbeacon(A) = 1:length(A);
  %every landmark floods the network
  lm_broadcast_snd = length(A)*n;
  lm_broadcast_rcv = length(A)*sum(out_degree);
  
  if debug
    fprintf('# broadcast packets sent: %d\n', lm_broadcast_snd);
    fprintf('# broadcast packets received: %d\n', lm_broadcast_rcv);
  end

  [beacon_vectors, nexthop2bcn] = allpair_dijkstra(neighbor_dist, A);
  [deltaA, cent] = min(beacon_vectors,[],2);
  %plot_cumu_matrix(deltaA);%!!!!
  if debug
    fprintf('mean and max deltaA:%.1f, %.1f\n', mean(deltaA), ...
	    max(deltaA));
  end
  if debug & n<=50
    deltaA'
  end
  
  %%%now get treeLabel of each node
  fprintf('CR: calculating tree labels...\n');
  tlabel = cell(n,1);
  for i=ordinary_nodes
    bcn = cent(i);
    if bcn<=0
      error('check here');
    end
    
    if deltaA(i)==Inf
      tlabel{i}(1) = -1;
    else
      curr = i;
      for count = deltaA(i)-1:-1:1
        curr = nexthop2bcn(curr, bcn);
        tlabel{i}(count) = curr;
      end
    end
  end
  %%%
  
  %now sort out the clusters for ordinary nodes
  %%C = delta < repmat(deltaA,n,1);
  %%C = delta <= repmat(deltaA,n,1); %!!!change from the algorithm
  %%C(:,A) = zeros(n, length(A));
  %%C(A,:) = zeros(length(A),n);
  if reactive==1
    fprintf('reactive mode=1, do NOT do local broadcasting\n')
  elseif reactive==0
    fprintf('simulating local broadcasting...\n');
    % every ordinary node i floods the network in the radix of deltaA(i) to let others find itself in their clusters
    od_broadcast_snd = 0;
    od_broadcast_rcv = 0;
    
    count = 0; total_c = length(ordinary_nodes); old_c = 0;
    start_t = clock;old_t = clock;
    
    for i=ordinary_nodes
      new_t = clock;
      if etime(new_t, old_t)>10
	fprintf('progress %.1f%%. ETA %.1f secs\n', count*100/total_c, ...
		etime(new_t,old_t)/(count-old_c)*(total_c-count));
	old_t = new_t;
	old_c = count;
      end
      
      [b_snd, b_rcv, rt_i,label(i),rt_dist ] = BFS(i,deltaA(i), neighbor_dist, ...
					   1, A(cent(i)) );
      routing_table(:,i) = rt_i';
      tmp = find(neighbor_dist(:,i));
      routing_table(tmp,i)=repmat(i,length(tmp),1);
      direct_dist(:,i) = rt_dist';
      direct_dist(tmp,i) = neighbor_dist(tmp,i);
      routing_table(n,n) = 0;
      od_broadcast_snd = od_broadcast_snd + b_snd;
      od_broadcast_rcv = od_broadcast_rcv + b_rcv;
      count = count+1;
    end
  
    if debug
      fprintf('# broadcast packets sent due to local clustering: %d\n', od_broadcast_snd);
      fprintf('# broadcast packets received due to local clustering: %d\n', od_broadcast_rcv);
    end
  elseif reactive==2
    %ok. distance vector mode
    fprintf('simulating scoped distance vector..\n');
    [dv, dv_packets, dv_entries] = distance_vector(neighbor_dist, deltaA);
    %update routing_table and direct_dist according to dv
    %rt = full(routing_table);
    %for i=1:n
    %  dests = find(dv{i}(:,1)>0);
    %  rt(i,dests) = dv{i}(dests,1)';
    %  rt(i,i) = 0;
    %end
    %routing_table = sparse(rt);
    %clear rt;
    routing_table = dv2rt(dv, routing_table);
    dd = full(direct_dist);
    for i=1:n
      dests = find(dv{i}(:,1)>0);
      dd(i,dests) = dv{i}(dests,2)';
    end
    direct_dist = sparse(dd);
    clear dd;
    clear dv;
  end
      
  %%%%
  if length(pre_landmarks)>0
    W = [];
  else
    W = find( sum(C,2) > 4*n/s);
  end
  
end
if debug
  fprintf('%d selected landmarks.\n',length(A));
  if n<100
    A
  end
end
%% end of selecting landmarks.
%% other useful variables: cent(dest), C(src,dest)


%%%%use DHT to store node coordinates, see the overhead
%storage overhead per beacon, + packets sent
fprintf('CR: putting vectors to beacons as a DHT\n');
beacons = A;
ordinary_nodes = setdiff(1:n, beacons);
storage_load = zeros(length(beacons),1);
dht_packets = 0;

if study_dht
  stats = struct('fail_total', 0, ...
		 'fail_due2ttl', 0, ...
		 'fail_due2prev',0, ...
		 'fail_before_beacon',0, ... %failed before reach the closest beacon
		 'fail_after_beacon', 0);
  for i=ordinary_nodes
    b_k = consistent_hash(i,beacons, n); %b_k \in 1.. length(beacons)
    storage_load(b_k) = storage_load(b_k)+1;
    tmp_dist = 0;
    [f_path, f_dist, routing_table, direct_dist, tmp_load, stats] = cr_routing(i, ...
						  A(b_k), cent, A, debug_route, beacon_vectors, neighbor_dist, ...
						  routing_table, nexthop2bcn, isbeacon, [],reactive, ...
						  deltaA, direct_dist, ...
						  zeros(n,1), ...
						  zeros(n,1), stats);
  
    [b_path, b_dist, routing_table, direct_dist, tmp_load, stats] = cr_routing(A(b_k), ...
						  i, cent, A, debug_route, beacon_vectors, neighbor_dist, ...
						  routing_table, nexthop2bcn, isbeacon, tlabel{i},reactive, ...
						  deltaA, direct_dist, ...
						  zeros(n,1), ...
						  zeros(n,1), stats);
    dht_packets = dht_packets + f_dist + b_dist;
  end
  if length(beacons)<10
    storage_load
  end
end
fprintf('CR: dht_packets:%d; mean storage load:%.2f, max:%.2f\n', ...
	 dht_packets, mean(storage_load), max(storage_load));
%%%%


%% routing algorithm
if 0 %old.
  routing_nexthop = optimal_nexthop(:, A(cent)); %route to the
						 %nearest landmarks
						 routing_nexthop(:, A) = optimal_nexthop(:, A); % routes to
												% landmarks are optimal
  for src = 1:n
    routing_nexthop(src, src) = src; %route to itself
    dest = src;
    routing_nexthop(A(cent(dest)),dest) = optimal_nexthop(A(cent(dest)),dest);
  end
  optimal_mask = C | (neighbor_dist>0); %routing within the cluster
                                        %and routing within
                                        %neighborhood is optimal
  routing_nexthop = optimal_nexthop.*optimal_mask + ...
      routing_nexthop.*(1-optimal_mask); % if C(src,dest)==1, next hop is the optimal one
end
routing_load = zeros(n,1);
routing_dist = zeros(length(test_pairs), 1);

%routing_table
stats = struct('fail_total', 0, ...
	       'fail_due2ttl', 0, ...
	       'fail_due2prev',0, ...
	       'fail_before_beacon',0, ... %failed before reach the closest beacon
	       'fail_after_beacon', 0);
for count = 1:length(test_pairs)
  src = test_pairs(count,1);
  dest = test_pairs(count,2); 
  [routing_path, routing_dist(count), routing_table, direct_dist, routing_load, stats] = cr_routing(src, ...
	dest, cent, A, debug_route, beacon_vectors, neighbor_dist, ...
        routing_table, nexthop2bcn, isbeacon, tlabel{dest},reactive, ...
						  deltaA,direct_dist, ...
						  failure_nodes, ...
						  routing_load, stats);
  %routing_load(routing_path)=  routing_load(routing_path)+1;
  if 0 & routing_dist(count)==Inf & length(routing_path)>1 %for  %debugging only
    plot_route(coord, routing_path,'b');
    plot_route(coord, [src A(cent(dest)) dest],'r');
    error('stop here')
  end
end

% check out the overhead
cluster = full(sum(routing_table>0,2));
if debug
  fprintf('mean and max cluster size:%.1f, %.1f\n', mean(cluster), ...
	  max(cluster));
  %[tmp idx] = max(cluster);
  %deltaA(idx)
  %beacon_vectors(idx,:)
  if n<100
    fprintf('cluster size for each node:');
    cluster'
    fprintf('total cluster size:%d\n',sum(cluster));
  end
  stats
end


function [routing_path, routing_dist, routing_table, direct_dist, routing_load, ...
	 stats ] =cr_routing(src, ...
	dest, cent, A, debug_route, beacon_vectors, neighbor_dist, ...
	  routing_table, nexthop2bcn, isbeacon, label, reactive, ...
	  deltaA, direct_dist, failure_nodes, routing_load, stats)
MAX_ROUTE_HOPS = 1000;
routing_path = [src];
routing_dist = 0;
hops =1;
failed_hops = 0; %# of hops that need failure handling
max_consecutive_fail = 0;
found_loop = 0;
if src==dest
  routing_dist = 0;
  return
end
if nargin<17
  stats = struct('fail_total', ...
		 'fail_due2ttl', 0, ...
		 'fail_due2prev',0, ...
		 'fail_before_beacon',0, ... %failed before reach the closest beacon
		 'fail_after_beacon', 0);
end
n = length(neighbor_dist);
k=5;
header = zeros(k,3); %packet annotation: k1 node's info:
                     %format(id,range,dist) range+dist==deltaA(id)
cent_dest = A(cent(dest)); %the closest landmark to dest
if debug_route
  fprintf('%d->%d:', src, dest);
end

if beacon_vectors(dest, cent(dest))==Inf ...
      | beacon_vectors(src, cent(dest))==Inf
  %not reachable
  routing_dist = Inf;
  fprintf('unreachable\n');
  stats.fail_total = stats.fail_total;
  return
end
if failure_nodes(src) | failure_nodes(dest)
  routing_dist = Inf;
  routing_path = [];
  if debug_route
    fprintf('\n');
  end
  fprintf('source or dest failure\n');
  return
end
curr = src;
weirdo = 0; %debug routing failure
%routing_load(curr) = routing_load(curr) +1; 
prev = -1;
passed_beacon = 0;
est_dist = Inf; %estimated distance to the destination
best_est_dist = Inf;
ttl = (deltaA(dest)+beacon_vectors(curr,cent(dest)))+10;
best_total_dist = (deltaA(dest)+beacon_vectors(curr,cent(dest)));

while curr~=dest
  
  %now the packet arrives at node curr
  tmpid = [];
  redirected_by_bcn = 0;
  if reactive==1 & length(label)>0
    tmpid = find(label==curr);
  end
  
%  if neighbor_dist(curr, dest)>0;
%    next = dest;
%    est_dist = 1;
%    if debug_route
%      fprintf('->n%d[1]', next);
%    end
  if routing_table(curr,dest)>0
    next = routing_table(curr, dest);
    est_dist = direct_dist(curr, dest);
    passed_beacon = 1;
    if debug_route
      fprintf('->r%d[%d]', next,direct_dist(curr,dest));
      if direct_dist(curr,dest)<=0
	fprintf('??????');
      end
    end
  elseif isbeacon(dest)
    next = nexthop2bcn(curr,isbeacon(dest));
    est_dist = beacon_vectors(curr,isbeacon(dest));
    if debug_route
      fprintf('->B%d[%d]', next, beacon_vectors(curr,isbeacon(dest)));
    end
  elseif curr ==cent_dest
    next = label(1);
    redirected_by_bcn = 1;
    passed_beacon = 1;
    est_dist = deltaA(dest);
    fprintf('routing via label\n');%%shouldn't happen when the
                                   %range is not typical CR
    if debug_route
      fprintf('->L%d[%d]', next, deltaA(dest));
    end
  elseif reactive==1 & length(tmpid)==1
    next = label(tmpid+1);
    est_dist = length(label)-tmpid+1;
    if debug_route
      fprintf('->LL%d[?]', next);
    end
  else%routing towards the beacon
    next = nexthop2bcn(curr, cent(dest));
    est_dist = deltaA(dest)+beacon_vectors(curr,cent(dest));
    if passed_beacon==1 & ~weirdo
      fprintf('weird. we should not have passed the beacon\n');
      weirdo = 1;
    end
    
    if debug_route
      fprintf('->b%d[[%d]]', next,deltaA(dest)+beacon_vectors(curr,cent(dest)));
    end
  end
  best_est_dist = min(best_est_dist, est_dist);
  best_total_dist = min(best_total_dist, est_dist+hops-1);
  
  if 0 & (est_dist+hops-1-best_total_dist>=3) & failure_nodes(next)
    fprintf('wasted hops:%d\n',est_dist+hops-1-best_total_dist);
    next = -1; %this one doesn't work well
    routing_dist = Inf;
    break;
  end
  % we got a potential next hop, but haven't sent the packet yet.
  if failure_nodes(next) | (next==prev)
    failed_hops = failed_hops + 1;
    max_consecutive_fail = max(max_consecutive_fail,failed_hops);
    if debug_route
      fprintf('->DEAD(%d)',next);
      fprintf('{%d}',max_consecutive_fail);
    end
    failover_nexts = find(neighbor_dist(curr,:)>0 & ~ ...
			  failure_nodes');
    failover_nexts = setdiff(failover_nexts, [prev]);
    l = length(failover_nexts);
    
    next_distance = repmat(Inf,l,1);
    dead_node = next; %will be transmitted to neighbors for reference
    next = prev;
    if l==0
      if debug_route
	fprintf('ALLDEAD')
      end
    else
      %imagine curr node is broadcasting the local recovery request
      routing_load(curr) = routing_load(curr) + 1;
      routing_dist = routing_dist + 1;
    end
    
    for i=1:l
      next = failover_nexts(i);
      next2 = -1;
      tmpid = [];
      redirected_by_bcn = 0;
      if reactive==1 & length(label)>0
	tmpid = find(label==next);
      end
      if routing_table(next,dest)>0
	next_distance(i) = direct_dist(next, dest);
	next2 = routing_table(next,dest);
      elseif isbeacon(dest)
	next_distance(i) = beacon_vectors(next, isbeacon(dest));
	next2 = nexthop2bcn(next, isbeacon(dest));
      elseif next==cent_dest
	next_distance(i) = deltaA(dest);
	next2 = label(1);
      elseif reactive==1 & length(tmpid)==1
	next_distance(i) = length(label)-tmpid+1;
	next2 = label(tmpid+1);
      else % route via beacon
	next_distance(i) = deltaA(dest)+beacon_vectors(next, ...
						       cent(dest));
	next2 = nexthop2bcn(next, cent(dest));
      end
      if failure_nodes(next2)
	%next_distance(i) = MAX_ROUTE_HOPS; %don't route it
	next_distance(i) = next_distance(i) + 0.5;
      end
      if next2==curr
	next_distance(i) = MAX_ROUTE_HOPS;
      end
      
    end %for each live neighbor
    if l>0
      
      %the neighbors will respond to the sender in the order of
      %next_distance. if it's the same, break it by a random number
      next_distance_weight = next_distance + rand(l,1)*0.1;
      ack_to_send = zeros(n,1);
      ack_to_send(failover_nexts) = ones(l,1);
      next = -1;
      while length(find(ack_to_send)>0)
	[tmp_dist min_idx] = min(next_distance_weight);
	next_distance_weight(min_idx) = Inf;
	tmp_dist = next_distance(min_idx);
	%if est_dist - tmp_dist<-0.6%!!!!
	%  break
	%end
	
	ack_node = failover_nexts(min_idx);
	if next<0 %no one has acked yet
	  if tmp_dist >= MAX_ROUTE_HOPS
	    %all nodes can't reach the dest
	    %nobody does broadcast
	    if debug_route
	      fprintf('->DEADEND');
	    end
	    break;
	  end
	  next = ack_node;
	  if debug_route
	    fprintf('->try(%d:%.1f)',next,est_dist-tmp_dist);
	  end
	end
	if tmp_dist>=MAX_ROUTE_HOPS %ok, the rest of the nodes will not reply
	  break
	end
	if ~ack_to_send(ack_node)
	  %fprintf('saved one bd\n');
	  continue %don't send broadcast
	end
	routing_load(ack_node) = routing_load(ack_node) + 1;
	routing_dist = routing_dist + 1;
	%hops = hops +1;
	%routing_path(hops) = ack_node;
	%other nodes that receive this ack should not ack again
	nodes_recv_ack = find(neighbor_dist(ack_node,:));
	ack_to_send(nodes_recv_ack) = zeros(length(nodes_recv_ack), ...
					    1);
	ack_to_send(ack_node) = 0;
      end
    else %no neighbor except prev available, how sad
      stats.fail_due2prev = stats.fail_due2prev + 1;
      if next~=prev
	error('absurd.');
      end
      if debug_route
	fprintf('->back2prev');
      end
      next = -1;
    end
  else %not failed nexthop
    %max_consecutive_fail = max(max_consecutive_fail,failed_hops);
    failed_hops = 0; %clear it
  end % end of if failure
  
  ttl = ttl -1;
  if ttl<0 | next<0
    routing_dist = Inf;
    if ttl<0
      stats.fail_due2ttl = stats.fail_due2ttl + 1;
    end
    if debug_route & ttl<0
      fprintf('TTLexp');
    end
    break;
  end
  
  if reactive==1 
    if redirected_by_bcn
      if header(k,2)>0
	error('something impossible happened');
      end
      %clear all headers anyway because path may not be optimal
      header(:,1) = zeros(k,1);
    end
    %update the header
    if deltaA(curr)>neighbor_dist(curr, next)
      header(1,:) = [curr deltaA(curr) 0]; 
    end
  end
  
  if ismember(next, routing_path) & ~found_loop
    found_loop =1;
    %fprintf('found loop and exit as failing\n');
    next = -1;
    routing_dist = Inf;
    break;
  end
    
  prev = curr;
  curr = next;
  %the actual packet got transmitted here to the next hop

  hops = hops +1;
  routing_path(hops) = next;
  routing_load(prev) = routing_load(prev) +1;
  routing_dist = routing_dist+neighbor_dist(prev, curr);
    
  if reactive==1%upon receiving the packet, update routing table
    %update the distances in the header, and sort it
    header(:,2) = header(:,2)-neighbor_dist(prev, curr);
    header(:,3) = header(:,3)+neighbor_dist(prev, curr);
    [tmp idx] = sort(header(:,2));
    header = header(idx,:);
    
    in_cluster_id = find(header(:,2)>=0 & header(:,1)>0); %!!>= is
                                                          %not
                                                          %exactly CR
    %in_cluster_id = find(header(:,2)>0 & header(:,1)>0);
    l = length(in_cluster_id);
    if l>0
    %if 0 %!!!!!!!!!!!!!!!
      routing_table(curr, header(in_cluster_id,1)) = ...
 	  repmat(prev, 1, l);
      direct_dist(curr, header(in_cluster_id,1)) = header(in_cluster_id,3)';
%      if debug_route
      if 0
	fprintf('updating routing table of %d: nexthop:%d,dests:',curr, ...
		routing_path(hops-1));
	header(in_cluster_id,1)
      end
      
    end
  end  
end

if routing_dist ==Inf
  if passed_beacon 
    stats.fail_after_beacon = stats.fail_after_beacon + 1;
  else
    stats.fail_before_beacon = stats.fail_before_beacon + 1;
  end
  stats.fail_total = stats.fail_total + 1;
  if 0 & ttl<0 
    max_consecutive_fail, hops
  end
  if weirdo
    fprintf('failed, and weirdo==1\n');
  end
  if found_loop
    %fprintf('failed, and found_loop==1\n');
  end
  %est_dist - best_est_dist
end
if 0 & max_consecutive_fail > 2
  fprintf('max_consecutive_fail=%d, failed=%d\n',max_consecutive_fail, ...
	  routing_dist == Inf);
end
if debug_route
  fprintf('\n');
end


function S = sample(W, s)
%return a random subset of W by selecting each element,
%independently with probability s/|W|

n = length(W);
if n<=s
  S = W;
  return
end
idx = find(rand(1,n) < s/n);
S = W(idx);
return

function [b_snd, b_rcv, nexthop,label,d_dist] = BFS(start_node, range, neighbor_dist, ...
			       min_dist, cent)
% use breadth first search to simulate broadcasting a packet from a
% node within certain range
% BFS is more realistic than DFS because the nature of broadcasting
% min_dist: the minimal distance that two nodes could possibly be,
% 1 if it's hop count, must be >=0
% b_snd: how many broadcast packets are sent
% b_rcv: how many broadcast packets are received
% nexthop(i): to start_node
% cent: closest beacon to start_node
% label: =nexthop(cent,start_node)
% d_dist(i): distance between i and start_node if i is in routing_table
label = 0;
queue = [[start_node, range, 0]];
b_snd = 0;
b_rcv = 0;
nexthop = sparse([]);
n = length(neighbor_dist);
nexthop(n) = 0;
d_dist = nexthop;
if range<=min_dist
  return
end


shortest_dist = repmat(Inf, 1, n); % the initial shortest distance to node
shortest_dist(start_node) = 0; %except to itself
%fprintf('start brdcst(cent%d):',cent);
while length(queue)>0
  %dequeue
  current = queue(1,:);
  node = current(1); range = current(2); dist = current(3);
  [l,tmp] = size(queue);
  queue = queue(2:l,:);

  %fprintf('->%d',node);
  
  %broadcast on current node
  neighbors = find(neighbor_dist(node,:));
  b_snd = b_snd + 1;
  b_rcv = b_rcv + length(neighbors); % all neighbors will hear it
  for i=1:length(neighbors)
    dst = neighbors(i);
    if dst==node
      continue
    elseif dst==cent
      label = node;
    end
    if range-neighbor_dist(node, dst)<min_dist
      continue
    end
    if dist+neighbor_dist(node,dst)<shortest_dist(dst)
      shortest_dist(dst) = dist+neighbor_dist(node,dst);
%      fprintf('%d->%d---->%d\n',dst, node, start_node);
      nexthop(dst) = node;
      [l,tmp] = size(queue);
      queue(l+1,:) = [dst, range-neighbor_dist(node, dst),dist+ ...
		      neighbor_dist(node,dst)];
    end
  end
end
idx = find(shortest_dist<Inf);
d_dist(idx) = shortest_dist(idx);
%fprintf('\n');