function  [routing_dist, routing_load, storage_load, dht_packets, cluster] = ...
    cr_bf(neighbor_dist,pre_landmarks, ...
		   test_pairs, failure_nodes, reactive)
%Refer to "Compact routing schemes" by Thorup & Zwick for the
%algorithm 
%neighbor_dist: distances are arbitrary ones, not necessarily
%hops. inf indicates non-connectivity.
% 
% pre_landmarks: pre-set landmark sets. 
% if not given, used the proposed randomized algorithm in the paper
% if length(pre_landmarks)==1, it's used as the number of landmarks

debug = 1;
debug_route = 0;
if ~issparse(neighbor_dist)
  neighbor_dist = dist_full2sparse(neighbor_dist);
end
if nargin<5
  reactive = 1; %default CR is proactive, i.e. reactive = 0;
end

n = length(neighbor_dist);
%fprintf('calculating best path...');
%[delta optimal_nexthop] = floyd_warshall(neighbor_dist);
%[delta optimal_nexthop] = allpair_dijkstra(neighbor_dist);
%fprintf('done\n');

%%%select landmarks
W = 1:n;
if nargin<2
  pre_landmarks = []; 
end

if nargin<3
  tmp1 = repmat(1:n,n,1);
  tmp2 = tmp1';
  rest_pairs = [tmp1(:) tmp2(:)];
end

if nargin<4
  failure_nodes = [];
end

fprintf('CR: reactive mode:%d\n',reactive);

out_degree = full(sum(neighbor_dist>0,2));
%C = zeros(n,n); %C(w,v)==1 means v \in C(w)
%routing_table(w,v)=nexthop, if v \in C(w)
routing_table = sparse([]);
routing_table(n,n) = 0;
direct_dist = routing_table; %should be sync with routing_table:
                             %direct_dist(w,v)=shortest_dist bwteen w&v

%nexthop2bcn(i,b) = nexthop(i,b) where b is the relative beacon id
deltaA = zeros(1,n); % deltaA(v) == \delta(A,v)
%cent(u), the closest landmark to each node, relative index to A

A = [];
s = sqrt(n*log(n)/log(2));
%s = sqrt(n);%!!!!
while length(W)>0
  if length(pre_landmarks)>=1%!!!!
    A = pre_landmarks;
  elseif length(pre_landmarks)==1
    % pre_landmarks is defined as the # of random landmarks
    tmp = randperm(n);
    A = tmp(1:pre_landmarks);
  else
    A = union(A, sample(W,s));
  end
  A = sort(A);
  ordinary_nodes = setdiff(1:n, A);
  isbeacon = zeros(n,1);
  isbeacon(A) = 1:length(A);
  %every landmark floods the network
  lm_broadcast_snd = length(A)*n;
  lm_broadcast_rcv = length(A)*sum(out_degree);
  
  if debug
    fprintf('# broadcast packets sent: %d\n', lm_broadcast_snd);
    fprintf('# broadcast packets received: %d\n', lm_broadcast_rcv);
  end

  [beacon_vectors, nexthop2bcn] = allpair_dijkstra(neighbor_dist, A);
  [deltaA, cent] = min(beacon_vectors,[],2);
  %plot_cumu_matrix(deltaA);%!!!!
  if debug
    fprintf('mean and max deltaA:%.1f, %.1f\n', mean(deltaA), ...
	    max(deltaA));
  end
  if debug & n<=50
    deltaA'
  end
  
  %%%now get treeLabel of each node
  fprintf('CR: calculating tree labels...\n');
  tlabel = cell(n,1);
  for i=ordinary_nodes
    bcn = cent(i);
    if bcn<=0
      error('check here');
    end
    
    if deltaA(i)==Inf
      tlabel{i}(1) = -1;
    else
      curr = i;
      for count = deltaA(i)-1:-1:1
        curr = nexthop2bcn(curr, bcn);
        tlabel{i}(count) = curr;
      end
    end
  end
  %%%
  
  %now sort out the clusters for ordinary nodes
  %%C = delta < repmat(deltaA,n,1);
  %%C = delta <= repmat(deltaA,n,1); %!!!change from the algorithm
  %%C(:,A) = zeros(n, length(A));
  %%C(A,:) = zeros(length(A),n);
  if reactive
    fprintf('do NOT do local broadcasting\n')
  else
    fprintf('simulating local broadcasting...\n');
    % every ordinary node i floods the network in the radix of deltaA(i) to let others find itself in their clusters
    od_broadcast_snd = 0;
    od_broadcast_rcv = 0;
    
    count = 0; total_c = length(ordinary_nodes); old_c = 0;
    start_t = clock;old_t = clock;
  
    for i=ordinary_nodes
      new_t = clock;
      if etime(new_t, old_t)>10
	fprintf('progress %.1f%%. ETA %.1f secs\n', count*100/total_c, ...
		etime(new_t,old_t)/(count-old_c)*(total_c-count));
	old_t = new_t;
	old_c = count;
      end
      
      [b_snd, b_rcv, rt_i,label(i),rt_dist ] = BFS(i,deltaA(i), neighbor_dist, ...
					   1, A(cent(i)) );
      routing_table(:,i) = rt_i';
      direct_dist(:,i) = rt_dist';
      
      routing_table(n,n) = 0;
      od_broadcast_snd = od_broadcast_snd + b_snd;
      od_broadcast_rcv = od_broadcast_rcv + b_rcv;
      count = count+1;
    end
  
    if debug
      fprintf('# broadcast packets sent due to local clustering: %d\n', od_broadcast_snd);
      fprintf('# broadcast packets received due to local clustering: %d\n', od_broadcast_rcv);
    end
  end
  %%%%
  if length(pre_landmarks)>0
    W = [];
  else
    W = find( sum(C,2) > 4*n/s);
  end
  
end
if debug
  fprintf('%d selected landmarks.\n',length(A));
  if n<100
    A
  end
end
%% end of selecting landmarks.
%% other useful variables: cent(dest), C(src,dest)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% use bloom filter to store routing information

%% merge routing_table, next2bcn and tlabel 

% rtb{i} is the routing table at node i
% each entry has two elements, dest and nexthop 
rtb = cell(n,1);

% from routing_table
for i=1:n
  v = find(routing_table(i,:)>0)';
  rtb{i} = [rtb{i}; [v routing_table(i,v)']];
end

% from nexthop2bcn
for i=1:n
  b = find(nexthop2bcn(i,:)>0)';
  rtb{i} = [rtb{i}; [A(b)' nexthop2bcn(i,b)']];
end

% from tlabel
for i=1:n
  if ~isempty(tlabel{i}) && tlabel{i}(1) > 0
    b = A(cent(i));	% landmark closest to i
    rtb{b} = [rtb{b}; [i tlabel{i}(1)]];
  end
end

%% build routing bloom filters

% rbf{i} is the bloom filter at node i
rbf = cell(n,1);

% adapted from Universal Class Hash Functions
ratio = 15;	% ratio of the bit filter size to the numbet of keys
hk = 8;		% number of independent hash functions
%p = 10007;	% smallest prime number greater than 10000
p = [10007 10009 10037 10039 10061 10067 10069 10079];
%coe1 = floor(rand([1 hk])*(p-2))+1;	0<coe1<p
%coe2 = floor(rand([1 hk])*(p-1));	0<=coe2<p
coe1 = [2092 3801 7838 6812 4614 5682 7947 593];
coe2 = [6032 502 4156 3051 8748 150 7684 9714];

for i=1:n
  %rtb{i}
  bitlength = size(rtb{i},1)*ratio;
  rbf{i} = zeros(bitlength,1);
  for j=1:hk
    bitpos = mod(mod(coe1(j)*rtb{i}(:,1) + coe2(j)*rtb{i}(:,2),p(j)),bitlength);
    rbf{i}(bitpos+1) = 1;
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%use DHT to store node coordinates, see the overhead
%storage overhead per beacon, + packets sent
fprintf('CR: putting vectors to beacons as a DHT\n');
beacons = A;
ordinary_nodes = setdiff(1:n, beacons);
storage_load = zeros(length(beacons),1);
dht_packets = 0;
for i=ordinary_nodes
  b_k = consistent_hash(i,beacons, n); %b_k \in 1.. length(beacons)
  storage_load(b_k) = storage_load(b_k)+1;
  tmp_dist = 0;
  [f_path, f_dist, routing_table, direct_dist] = cr_routing(i, ...
					       A(b_k), cent, A, debug_route, beacon_vectors, neighbor_dist, ...
					       routing_table, nexthop2bcn, isbeacon, [],reactive, ...
					       deltaA, direct_dist, ...
						  failure_nodes);
  
  [b_path, b_dist, routing_table, direct_dist] = cr_routing(A(b_k), ...
					       i, cent, A, debug_route, beacon_vectors, neighbor_dist, ...
					       routing_table, nexthop2bcn, isbeacon, tlabel{i},reactive, ...
					       deltaA, direct_dist, ...
						  failure_nodes);
  dht_packets = dht_packets + f_dist + b_dist;
end
if length(beacons)<10
  storage_load
end

fprintf('CR: dht_packets:%d; mean storage load:%.2f, max:%.2f\n', ...
	 dht_packets, mean(storage_load), max(storage_load));
%%%%


%% routing algorithm
if 0 %old.
  routing_nexthop = optimal_nexthop(:, A(cent)); %route to the
						 %nearest landmarks
						 routing_nexthop(:, A) = optimal_nexthop(:, A); % routes to
												% landmarks are optimal
  for src = 1:n
    routing_nexthop(src, src) = src; %route to itself
    dest = src;
    routing_nexthop(A(cent(dest)),dest) = optimal_nexthop(A(cent(dest)),dest);
  end
  optimal_mask = C | (neighbor_dist>0); %routing within the cluster
                                        %and routing within
                                        %neighborhood is optimal
  routing_nexthop = optimal_nexthop.*optimal_mask + ...
      routing_nexthop.*(1-optimal_mask); % if C(src,dest)==1, next hop is the optimal one
end
routing_load = zeros(n,1);
routing_dist = zeros(length(test_pairs), 1);

for count = 1:length(test_pairs)
  src = test_pairs(count,1);
  dest = test_pairs(count,2); %@!@@@!!!!

%fprintf(1,'src %d, dest %d\n',src,dest);

%{
  [routing_path, routing_dist(count), routing_table, direct_dist] = cr_routing(src, ...
	dest, cent, A, debug_route, beacon_vectors, neighbor_dist, ...
        routing_table, nexthop2bcn, isbeacon, tlabel{dest},reactive, ...
						  deltaA,direct_dist, ...
						  failure_nodes);
%}
  [routing_path, routing_dist(count), direct_dist] = cr_bf_routing(src, ...
	dest, cent, A, debug_route, beacon_vectors, neighbor_dist, ...
        p, coe1, coe2, rbf, rtb, reactive, ...
						  deltaA,direct_dist, ...
						  failure_nodes);
  routing_load(routing_path)=  routing_load(routing_path)+1;
end

% check out the overhead
cluster = full(sum(routing_table>0,2));
if debug
  fprintf('mean and max cluster size:%.1f, %.1f\n', mean(cluster), ...
	  max(cluster));
  %[tmp idx] = max(cluster);
  %deltaA(idx)
  %beacon_vectors(idx,:)
  if n<100
    fprintf('cluster size for each node:');
    cluster'
    fprintf('total cluster size:%d\n',sum(cluster));
  end
end


function [routing_path, routing_dist, routing_table, direct_dist] =cr_routing(src, ...
	dest, cent, A, debug_route, beacon_vectors, neighbor_dist, ...
	  routing_table, nexthop2bcn, isbeacon, label, reactive, ...
	  deltaA, direct_dist, failure_nodes)
routing_path = [src];
routing_dist = 0;
hops =1;
if src==dest
  routing_dist = 0;
  return
end
k=5;
header = zeros(k,3); %packet annotation: k1 node's info:
                     %format(id,range,dist) range+dist==deltaA(id)
cent_dest = A(cent(dest)); %the closest landmark to dest
if debug_route
  fprintf('%d->%d:', src, dest);
end
  
if beacon_vectors(dest, cent(dest))==Inf | beacon_vectors(src, ...
						  cent(dest))==Inf
  %not reachable
  routing_dist = Inf;
  fprintf('unreachable\n');
  return
end
curr = src;
prev = -1;
passed_beacon = 0;
est_dist = Inf; %estimated distance to the destination

while curr~=dest
  tmpid = [];
  redirected_by_bcn = 0;
  if reactive & length(label)>0
    tmpid = find(label==curr);
  end
  
  if neighbor_dist(curr, dest)>0;
    next = dest;
    est_dist = 1;
    if debug_route
      fprintf('->n%d[1]', next);
    end
  elseif routing_table(curr,dest)>0
    next = routing_table(curr, dest);
    est_dist = direct_dist(curr, dest);
    if debug_route
      fprintf('->c%d[%d]', next,direct_dist(curr,dest));
      if direct_dist(curr,dest)<=0
	fprintf('??????');
      end
    end
  elseif isbeacon(dest)
    next = nexthop2bcn(curr,isbeacon(dest));
    est_dist = beacon_vectors(curr,isbeacon(dest));
    if debug_route
      fprintf('->B%d[%d]', next, beacon_vectors(curr,isbeacon(dest)));
    end
  elseif curr ==cent_dest
    next = label(1);
    redirected_by_bcn = 1;
    passed_beacon = 1;
    est_dist = deltaA(dest);
    if debug_route
      fprintf('->L%d[%d]', next, deltaA(dest));
    end
  elseif reactive & length(tmpid)==1
    next = label(tmpid+1);
    est_dist = length(label)-tmpid+1;
    if debug_route
      fprintf('->LL%d[?]', next);
    end
  else
    next = nexthop2bcn(curr, cent(dest));
    est_dist = deltaA(dest)+beacon_vectors(curr,cent(dest));
    if debug_route
      fprintf('->b%d[[%d]]', next,deltaA(dest)+beacon_vectors(curr,cent(dest)));
    end
  end
  hops = hops +1;
  routing_path(hops) = next;
  routing_dist = routing_dist+neighbor_dist(curr, next);
  
  if reactive 
    if redirected_by_bcn
      if header(k,2)>0
	error('something impossible happened');
      end
      %clear all headers anyway -- path may not be optimal
      header(:,1) = zeros(k,1);
    end
    %update the header
    if deltaA(curr)>neighbor_dist(curr, next)
      header(1,:) = [curr deltaA(curr) 0]; 
    end
    header(:,2) = header(:,2)-neighbor_dist(curr, next);
    header(:,3) = header(:,3)+neighbor_dist(curr, next);
    %expired_idx = find(header(:,2)<=0); 
    %expired_idx = find(header(:,2)<0); %!!!not compact routing
    %header(expired_idx,1) = zeros(length(expired_idx), 1);
    [tmp idx] = sort(header(:,2));
    header = header(idx,:);
  
  end
  prev = curr;
  curr = next;
  %the actual packet got transmitted here to the next hop
  
  if reactive%upon receiving the packet, update routing table
    in_cluster_id = find(header(:,2)>=0 & header(:,1)>0); %!!>= is
                                                          %not
                                                          %exactly CR
    %in_cluster_id = find(header(:,2)>0 & header(:,1)>0);
    l = length(in_cluster_id);
    if l>0
    %if 0 %!!!!!!!!!!!!!!!
      routing_table(curr, header(in_cluster_id,1)) = ...
 	  repmat(prev, 1, l);
      direct_dist(curr, header(in_cluster_id,1)) = header(in_cluster_id,3)';
%      if debug_route
      if 0
	fprintf('updating routing table of %d: nexthop:%d,dests:',curr, ...
		routing_path(hops-1));
	header(in_cluster_id,1)
      end
      
    end
  end  
end
if debug_route
  fprintf('\n');
end


function S = sample(W, s)
%return a random subset of W by selecting each element,
%independently with probability s/|W|

n = length(W);
if n<=s
  S = W;
  return
end
idx = find(rand(1,n) < s/n);
S = W(idx);
return

function [b_snd, b_rcv, nexthop,label,d_dist] = BFS(start_node, range, neighbor_dist, ...
			       min_dist, cent)
% use breadth first search to simulate broadcasting a packet from a
% node within certain range
% BFS is more realistic than DFS because the nature of broadcasting
% min_dist: the minimal distance that two nodes could possibly be,
% 1 if it's hop count, must be >=0
% b_snd: how many broadcast packets are sent
% b_rcv: how many broadcast packets are received
% nexthop(i): to start_node
% cent: closest beacon to start_node
% label: =nexthop(cent,start_node)
% d_dist(i): distance between i and start_node if i is in routing_table
label = 0;
queue = [[start_node, range, 0]];
b_snd = 0;
b_rcv = 0;
nexthop = sparse([]);
n = length(neighbor_dist);
nexthop(n) = 0;
d_dist = nexthop;
if range<=min_dist
  return
end


shortest_dist = repmat(Inf, 1, n); % the initial shortest distance to node
shortest_dist(start_node) = 0; %except to itself
%fprintf('start brdcst(cent%d):',cent);
while length(queue)>0
  %dequeue
  current = queue(1,:);
  node = current(1); range = current(2); dist = current(3);
  [l,tmp] = size(queue);
  queue = queue(2:l,:);

  %fprintf('->%d',node);
  
  %broadcast on current node
  neighbors = find(neighbor_dist(node,:));
  b_snd = b_snd + 1;
  b_rcv = b_rcv + length(neighbors); % all neighbors will hear it
  for i=1:length(neighbors)
    dst = neighbors(i);
    if dst==node
      continue
    elseif dst==cent
      label = node;
    end
    if range-neighbor_dist(node, dst)<min_dist
      continue
    end
    if dist+neighbor_dist(node,dst)<shortest_dist(dst)
      shortest_dist(dst) = dist+neighbor_dist(node,dst);
%      fprintf('%d->%d---->%d\n',dst, node, start_node);
      nexthop(dst) = node;
      [l,tmp] = size(queue);
      queue(l+1,:) = [dst, range-neighbor_dist(node, dst),dist+ ...
		      neighbor_dist(node,dst)];
    end
  end
end
idx = find(shortest_dist<Inf);
d_dist(idx) = shortest_dist(idx);
%fprintf('\n');
