function sim_parallel(instance, job_id)
% job_id: 
%  >0  -- called in condor for specific jobs, never use it manually
%  except for debugging!
%  ==0 -- called by user: run all jobs without using condor
% unspecified -- save consts and schedule condor jobs
% ==-1 -- don't calculate. only process and plot!

%supported instances:
% high_density
% low_density
% high_dht

fn = 'sim_parallel';
MAX_JOBS = 100;
if nargin==2
  if job_id == -1
    process_data(fn, instance)
  else
    get_raw_data(fn, instance, [job_id], MAX_JOBS)
  end
  return
end
total_jobs = get_raw_data(fn, instance, [], MAX_JOBS)
fid = fopen('condor.matlab','wt');
if fid<0
  error('could not open file');
end

fprintf(fid, '%s %s %d\n', fn, instance, min(total_jobs,MAX_JOBS));
fclose(fid);
! python gen_condor_jobs.py

function job_count = get_raw_data(fn, instance, job_ids, max_jobs)
if strcmp(instance,'high_density')
  topo = 'topo-3200-5.12v6.mat';
  n_beacons =[10, 20, 30, 56, 80, 100]
elseif strcmp(instance,'low_density')
  topo = 'topo-3200-3v6.mat';
  n_beacons =[10, 20, 30, 56, 80, 100]
elseif strcmp(instance,'very_low')
  topo = 'topo-1000.mat';
  n_beacons =[ 8, 16, 24, 32, 40]
elseif strcmp(instance,'high_dht')
  topo = 'topo-3200-5.12v6.mat';
  n_beacons =[10, 20, 30, 56, 80, 100]
else
  error('unknown topo file');
  %topo = 'topo-100-5.12-0.1.mat';
end
repeat = 10;
%repeat = 1;
%n_beacons =[10]
const_fn = sprintf('%s/%s-const.mat', instance, fn);

if length(job_ids)==0 | ismember(0, job_ids)
  const_fn = sprintf('%s/%s-const.mat', instance, fn);
  save(const_fn, 'repeat', 'n_beacons','topo','max_jobs','-V6');
end

load(const_fn); %make sure it's consistant between instances

job_count = 0;

S = load(topo);
%%%%%%% hetergeneous jobs.. :) the first one is optimal
job_count = job_count + 1;
if schedule_job(job_count, job_ids, max_jobs)
  fprintf('job 1\n');
  [optimal_dist optimal_nexthop] = allpair_dijkstra(S.D, ...
						    1:S.n_nodes, 1, ...
						    'optimal-cache.mat');
  load_optimal = get_nexthop_load(optimal_nexthop, S.random_pairs);
  final_fn = sprintf('%s/%s-%d.mat', instance, fn, job_count);
  save(final_fn, 'optimal_dist','load_optimal');
end
%%%%%%%
for i=1:repeat
  for b=1:length(n_beacons)
    job_count = job_count + 1;
    if schedule_job(job_count, job_ids, max_jobs)
      final_fn = sprintf('%s/%s-%d.mat', instance, fn, job_count);
      fprintf('**job_id=%d**sim2 test beacon=%d, i=%d\n', job_count, n_beacons(b), i);
      try
	tmp=load(final_fn);
	clear tmp
	fprintf('**ALREADY DID job_id=%d**sim2 test beacon=%d, i=%d\n', job_count, n_beacons(b), i);
      catch
	if strcmp(instance,'high_dht')
	  res = sim_with_dht(topo, n_beacons(b));
	elseif strcmp(instance,'very_low')
	  res = simulation(topo, n_beacons(b), 1);
	else
	  res = simulation(topo, n_beacons(b));
	end
	save(final_fn ,'res');
      end
    end
  end
end

function process_data(fn, instance)

load(sprintf('%s/%s-const.mat', instance,fn))
job_count = 1;
load(sprintf('%s/%s-1.mat', instance, fn))
for i=1:repeat
  for p=1:length(n_beacons)
    job_count = job_count + 1;
    final_fn = sprintf('%s/%s-%d.mat', instance, fn, job_count);
    TMP = load(final_fn);
    res(p,i) = TMP.res;
  end
end
save(sprintf('%s/%s-all.mat',instance,fn));

if strcmp(instance,'high_dht')
  sim_dht
  return
end
if strcmp(instance,'low_density')
  sim_density 
  % don't need to keep running and OVERWRITE the figures
  return
end

%%%for high_density and low_density
msz = 12; %MarkerSize
fontsize = 15;
INTERVAL=2;

%n_beacons =[10, 20, 56, 80, 100]
%repeat = 10;
topo
S = load(topo);
n_beacons

%%%%figure 1.a routing success rate
for i=1:repeat
  for b=1:length(n_beacons)
    failure_bvr(b,i) = res(b,i).failure_bvr;
    failure_bvr_2h(b,i) = res(b,i).failure_bvr_2h;
  end
end
figure(1)
set(gca, 'FontSize',fontsize);
hold on
plot_errorbar(failure_bvr, n_beacons-0.5, 'x-r', msz,1);
plot_errorbar(failure_bvr_2h, n_beacons+0.5, 'v-k', msz,1);

plot(n_beacons, zeros(1, length(n_beacons)),'o-b', 'MarkerSize', msz)
xlabel('number of beacons K');
ylabel('routing failure rate');
ylim([0 0.35]);
legend('BVR w/o flooding', 'BVR 2 hop neighbor w/o flooding', 'S4');
saveas(gcf, 'failure_rate.eps','psc2')

%%%%%figure 1.b routing stretch vs K
pair_indices = S.random_pairs(:,1)+(S.random_pairs(:,2)-1)*S.n_nodes;
optimal_dist = optimal_dist(pair_indices);
fprintf('average optimal distance:%.2f\n',mean(optimal_dist));
failure_opt = length(find(optimal_dist==Inf))/S.n_random_pairs;

figure(10)
set(gca, 'FontSize',fontsize);
hold on
colors_bvr=['k--';'   ';'k--';'k-.'];
colors_bvr_nf=['r--';'   ';'r--';'r-.'];
colors_bvr_rate=['g  ';'   ';'   ';'g--'];
colors_cr=['b ';'  ';'  ';'b:'];
for b =1:length( n_beacons)
  for i=1:repeat
    routing_dist_bvr_sf(:,i)=res(b,i).routing_dist_bvr_sf;
    routing_dist_bvr_nf(:,i)=res(b,i).routing_hops_sf;
    routing_dist_cr(:,i)=res(b,i).routing_dist_cr;
    routing_dist_bvr_2h(:,i)=res(b,i).routing_dist_bvr_2h;
    routing_dist_bvr_2hnf(:,i)=res(b,i).routing_hops_2h;
  end
  if ismember(b, [1 4])
    worst_bvr(b) = plot_routing_stretch(routing_dist_bvr_sf, optimal_dist, colors_bvr(b,:),[100],1);
    worst_cr(b) = plot_routing_stretch(routing_dist_cr, optimal_dist, ...
				       colors_cr(b,:),[100],1);
    worst_bvr_nf(b) = plot_routing_stretch(routing_dist_bvr_nf, ...
					   optimal_dist, colors_bvr_nf(b,:), ...
					   [100],1,1);
  end
  avg_routing_stretch_bvr_sf(b) = plot_routing_stretch(...
      routing_dist_bvr_sf, optimal_dist,'',[-1],0,0);
  avg_routing_stretch_bvr_nf(b) = plot_routing_stretch(...
      routing_dist_bvr_nf, optimal_dist,'',[-1],0,1);
  avg_routing_stretch_cr(b) = plot_routing_stretch(...
      routing_dist_cr, optimal_dist,'',[-1],0,0);
  avg_routing_stretch_bvr_2h(b) = plot_routing_stretch(...
      routing_dist_bvr_2h, optimal_dist,'',[-1],0,0);
  avg_routing_stretch_bvr_2hnf(b) = plot_routing_stretch(...
      routing_dist_bvr_2hnf, optimal_dist,'',[-1],0,1);
  %plot_routing_stretch(transmission_dist_bvr_2h, optimal_dist, 'r',[],1);
  tmp = mean(failure_bvr,2);
  %plot([1 6],[1-tmp(b), 1-tmp(b)],colors_bvr_rate(b,:));
end
bvr_str1 = sprintf('BVR w/ flooding. K=10, worst stretch=%.1f',worst_bvr(1));
bvr_str2 = sprintf('BVR w/ flooding. K=56, worst stretch=%.1f',worst_bvr(4));
bvr_nf_str1 = sprintf('BVR w/o flooding. K=10, worst stretch=%.1f',worst_bvr_nf(1));
bvr_nf_str2 = sprintf('BVR w/o flooding. K=56, worst stretch=%.1f',worst_bvr_nf(4));
cr_str1 = sprintf('S4. K=10, worst stretch=%.1f',worst_cr(1));
cr_str2 = sprintf('S4. K=56, worst stretch=%.1f',round(worst_cr(4)));
%legend(bvr_str1, cr_str1,bvr_nf_str1, 'BVR success rate K=10', ...
%       bvr_str2, ...
%       cr_str2, bvr_nf_str2,'BVR success rate K=56',0);
legend(bvr_str1, cr_str1,bvr_nf_str1,  ...
       bvr_str2, ...
       cr_str2, bvr_nf_str2,4);
xlabel('routing stretch');
ylabel('CDF');
xlim([1 3])
saveas(gcf, 'rstretch_normal.eps','psc2')

figure(21)
hold on
set(gca, 'FontSize',fontsize);
plot(n_beacons, avg_routing_stretch_cr, 'o-b', 'MarkerSize', msz);
plot(n_beacons, avg_routing_stretch_bvr_nf, '+-r', 'MarkerSize', msz);
plot(n_beacons, avg_routing_stretch_bvr_sf, 'x:r', 'MarkerSize', msz);
plot(n_beacons, avg_routing_stretch_bvr_2hnf, 'v-k', 'MarkerSize', msz);
plot(n_beacons, avg_routing_stretch_bvr_2h, '<:k', 'MarkerSize', msz);
xlabel('number of beacons K');
ylabel('average routing stretch');
legend('S4','BVR no flooding','BVR w/ flooding','BVR 2-hop no flooding',...
       'BVR 2-hop w/ flooding');
saveas(gcf, 'avg_routing_stretch.eps', 'psc2');

%%%%%%%%%%%%%%%%%%%%routing stretch 2 hop NEW
figure(11)
set(gca, 'FontSize',fontsize);
hold on
colors_bvr=['k--';'   ';'k--';'k-.'];
colors_bvr_nf=['r--';'   ';'r--';'r-.'];
colors_bvr_rate=['g  ';'   ';'   ';'g--'];
colors_cr=['b ';'  ';'  ';'b:'];
for b = [1,4]
  for i=1:repeat
    routing_dist_bvr_2h(:,i)=res(b,i).routing_dist_bvr_2h;
    routing_dist_bvr_sf(:,i)=res(b,i).routing_dist_bvr_sf;
    routing_dist_bvr_nf(:,i)=res(b,i).routing_hops_sf;
    routing_dist_cr(:,i)=res(b,i).routing_dist_cr;
  end
  worst_bvr(b) = plot_routing_stretch(routing_dist_bvr_2h, optimal_dist, colors_bvr(b,:),[100],1);
  worst_cr(b) = plot_routing_stretch(routing_dist_cr, optimal_dist, ...
  				 colors_cr(b,:),[100],1);
  %plot_routing_stretch(transmission_dist_bvr_2h, optimal_dist, 'r',[],1);
  tmp = mean(failure_bvr,2);
  %plot([1 6],[1-tmp(b), 1-tmp(b)],colors_bvr_rate(b,:));
end
bvr_str1 = sprintf('BVR 2hop w/ flooding. K=10, worst stretch=%.1f',worst_bvr(1));
bvr_str2 = sprintf('BVR 2hop w/ flooding. K=56, worst stretch=%.1f',worst_bvr(4));
cr_str1 = sprintf('S4. K=10, worst stretch=%.1f',worst_cr(1));
cr_str2 = sprintf('S4. K=56, worst stretch=%.1f',round(worst_cr(4)));
%legend(bvr_str1, cr_str1,bvr_nf_str1, 'BVR success rate K=10', ...
%       bvr_str2, ...
%       cr_str2, bvr_nf_str2,'BVR success rate K=56',0);
legend(bvr_str1, cr_str1, ...
       bvr_str2, ...
       cr_str2, 4);
xlabel('average routing stretch');
ylabel('CDF');
xlim([1 3])
saveas(gcf, '2hoprstretch_normal.eps','psc2')

%%%%%distance vs stretch%%%%%%%%%
figure(31)
hold on
set(gca, 'FontSize',fontsize);
b=4;
clear routing_dist_bvr_sf
for i=1:repeat
  routing_dist_bvr_sf(:,i) = res(b,i).routing_dist_bvr_sf;
  routing_dist_cr(:,i) = res(b,i).routing_dist_cr;
  %routing_dist_bvr_sf(:,i) = res(b,i).transmission_dist_bvr_sf;
  %routing_dist_cr(:,i) = res(b,i).routing_dist_cr;
end
plot_stretch_by_distance(routing_dist_bvr_sf, optimal_dist, 'r-x', ...
			 'relative', msz-3)
plot_stretch_by_distance(routing_dist_cr, optimal_dist, 'b-o','relative',msz-3)
xlabel('optimal routing distance (hops)')
ylabel('routing stretch')
legend('BVR','S4')
saveas(gcf, 'dist_stretch.eps','psc2')
xlim([1 43])
figure(32)
hold on
set(gca, 'FontSize',fontsize);
plot_stretch_by_distance(routing_dist_bvr_sf, optimal_dist, 'r-x', ...
			 'abs', msz-3)
plot_stretch_by_distance(routing_dist_cr, optimal_dist, 'b-o', 'abs', ...
			 msz-3)
xlabel('optimal routing distance (hops)')
ylabel('absolute routing distance error (hops) ')
legend('BVR','S4',2)
saveas(gcf, 'dist_abs_stretch.eps','psc2')
xlim([1 43])


%%%%%figure 1.b transmission stretch vs K
%pair_indices = S.random_pairs(:,1)+(S.random_pairs(:,2)-1)*S.n_nodes;
%optimal_dist = optimal_dist(pair_indices);
%failure_opt = length(find(optimal_dist==Inf))/S.n_random_pairs;

figure(2)
set(gca, 'FontSize',fontsize);
hold on
colors_bvr=['k--';'   ';'k--';'k-.'];
colors_bvr_rate=['g  ';'   ';'   ';'g--'];
colors_cr=['b ';'  ';'  ';'b:'];
for b = 1:length(n_beacons)
  for i=1:repeat
    transmission_dist_bvr_2h(:,i)=res(b,i).transmission_dist_bvr_2h;
    transmission_dist_bvr_sf(:,i)=res(b,i).transmission_dist_bvr_sf;
    transmission_dist_cr(:,i)=res(b,i).routing_dist_cr;
  end
  if ismember(b, [1 4])
    worst_bvr(b) = plot_routing_stretch(transmission_dist_bvr_sf, optimal_dist, colors_bvr(b,:),[100],1);
    worst_cr(b) = plot_routing_stretch(transmission_dist_cr, optimal_dist, ...
				       colors_cr(b,:),[100],1);
    tmp = mean(failure_bvr,2);
    plot([1 6],[1-tmp(b), 1-tmp(b)],colors_bvr_rate(b,:));
  end
  avg_tran_stretch_bvr_sf(b)=plot_routing_stretch(...
      transmission_dist_bvr_sf, optimal_dist, '',[-1],0);
  avg_tran_stretch_bvr_2h(b)=plot_routing_stretch(...
      transmission_dist_bvr_2h, optimal_dist, '',[-1],0);
  avg_tran_stretch_cr(b)=plot_routing_stretch(...
      transmission_dist_cr, optimal_dist, '',[-1],0);
  %plot_routing_stretch(transmission_dist_bvr_2h, optimal_dist, 'r',[],1);
end
bvr_str1 = sprintf('BVR w/ flooding. K=10, worst stretch=%.1f',worst_bvr(1));
bvr_str2 = sprintf('BVR w/ flooding. K=56, worst stretch=%.1f',worst_bvr(4));
cr_str1 = sprintf('S4. K=10, worst stretch=%.1f',worst_cr(1));
cr_str2 = sprintf('S4. K=56, worst stretch=%.1f',round(worst_cr(4)));
legend(bvr_str1, cr_str1, 'BVR success rate K=10', ...
       bvr_str2, ...
       cr_str2, 'BVR success rate K=56',0);
xlabel('transmission stretch');
ylabel('CDF');
xlim([1 6])
saveas(gcf, 'stretch_normal.eps','psc2')

figure(22)
hold on
set(gca, 'FontSize',fontsize);
plot(n_beacons, avg_tran_stretch_cr, 'o-b', 'MarkerSize', msz);
plot(n_beacons, avg_tran_stretch_bvr_sf, 'x-r', 'MarkerSize', msz);
plot(n_beacons, avg_tran_stretch_bvr_2h, 'v-k', 'MarkerSize', msz);
ylim([1 3.5]);
xlabel('number of beacons K');
ylabel('average transmission stretch');
legend('S4','BVR w/ flooding','BVR 2-hop w/ flooding');
saveas(gcf, 'avg_transmission_stretch.eps', 'psc2');

%%%%%%%%%%%%%%%2 hop transmission stretch NEW

figure(12)
set(gca, 'FontSize',fontsize);
hold on
colors_bvr=['k--';'   ';'k--';'k-.'];
colors_bvr_rate=['g  ';'   ';'   ';'g--'];
colors_cr=['b ';'  ';'  ';'b:'];
for b = [1,4]
  for i=1:repeat
    transmission_dist_bvr_2h(:,i)=res(b,i).transmission_dist_bvr_2h;
    transmission_dist_bvr_sf(:,i)=res(b,i).transmission_dist_bvr_sf;
    transmission_dist_cr(:,i)=res(b,i).routing_dist_cr;
  end
  worst_bvr(b) = plot_routing_stretch(transmission_dist_bvr_2h, optimal_dist, colors_bvr(b,:),[100],1);
  worst_cr(b) = plot_routing_stretch(transmission_dist_cr, optimal_dist, ...
				 colors_cr(b,:),[100],1);
  %plot_routing_stretch(transmission_dist_bvr_2h, optimal_dist, 'r',[],1);
  tmp = mean(failure_bvr_2h,2);
  plot([1 6],[1-tmp(b), 1-tmp(b)],colors_bvr_rate(b,:));
end
bvr_str1 = sprintf('BVR 2hop w/ flooding. K=10, worst stretch=%.1f',worst_bvr(1));
bvr_str2 = sprintf('BVR 2hop w/ flooding. K=56, worst stretch=%.1f',worst_bvr(4));
cr_str1 = sprintf('S4. K=10, worst stretch=%.1f',worst_cr(1));
cr_str2 = sprintf('S4. K=56, worst stretch=%.1f',round(worst_cr(4)));
legend(bvr_str1, cr_str1, 'BVR 2hop success rate K=10', ...
       bvr_str2, ...
       cr_str2, 'BVR 2hop success rate K=56',0);
xlabel('transmission stretch');
ylabel('CDF');
xlim([1 6])

saveas(gcf, '2hop_stretch_normal.eps','psc2')


%%%%%%%%figure 1.c,non-beacon node load
for i=1:repeat
  for b=1:length(n_beacons)
    non_beacons = setdiff(1:S.n_nodes, res(b,i).beacons);
    load_bvr_10(b,i) = percentile(res(b,i).load_bvr_sf(non_beacons), 10);
    load_bvr_50(b,i) = percentile(res(b,i).load_bvr_sf(non_beacons), 50);
    load_bvr_90(b,i) = percentile(res(b,i).load_bvr_sf(non_beacons), 90);

    load_bvr_2h_10(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 10);
    load_bvr_2h_50(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 50);
    load_bvr_2h_90(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 90);

    load_cr_10(b,i) = percentile(res(b,i).load_cr(non_beacons), 10);
    load_cr_50(b,i) = percentile(res(b,i).load_cr(non_beacons), 50);
    load_cr_90(b,i) = percentile(res(b,i).load_cr(non_beacons), 90);
  end
end
figure(3)
set(gca, 'FontSize',fontsize);
hold on
y = mean(load_bvr_50, 2);
l = y- mean(load_bvr_10, 2);
u = mean(load_bvr_90,2) - y;
if 1
  errorbar(n_beacons-INTERVAL, y, l, u, 'x-r','MarkerSize', msz);
else
  plot(n_beacons, y, 'x-r','MarkerSize', msz);
end

y = mean(load_bvr_2h_50, 2);
l = y- mean(load_bvr_2h_10, 2);
u = mean(load_bvr_2h_90,2) - y;
if 1
  errorbar(n_beacons+INTERVAL, y, l, u, 'v-k','MarkerSize', msz);
else
  plot(n_beacons, y, 'v-k','MarkerSize', msz);
end

y = mean(load_cr_50, 2);
l = y- mean(load_cr_10, 2);
u = mean(load_cr_90,2) - y;
if 1
  errorbar(n_beacons, y, l, u, 'o-b','MarkerSize', msz);
else
  plot(n_beacons, y, 'o-b','MarkerSize', msz);
end
xlim([0 105]);
ylim([0 1400]);
xlabel('number of beacons K');
ylabel('non-beacon node load');
legend('BVR', 'BVR 2-hop', 'S4');
saveas(gcf, 'nonbeacon_load.eps','psc2')

%%%%%%%%figure 1.c,non-beacon node load 2 hop NEW
if 0
  for i=1:repeat
    for b=1:length(n_beacons)
      non_beacons = setdiff(1:S.n_nodes, res(b,i).beacons);
      load_bvr_10(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 10);
      load_bvr_50(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 50);
      load_bvr_90(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 90);

      load_cr_10(b,i) = percentile(res(b,i).load_cr(non_beacons), 10);
      load_cr_50(b,i) = percentile(res(b,i).load_cr(non_beacons), 50);
      load_cr_90(b,i) = percentile(res(b,i).load_cr(non_beacons), 90);
    end
  end
  figure(13)
  set(gca, 'FontSize',fontsize);
  y = mean(load_bvr_50, 2);
  l = y- mean(load_bvr_10, 2);
  u = mean(load_bvr_90,2) - y;
  errorbar(n_beacons-INTERVAL, y, l, u,'x-r','MarkerSize', msz);
  hold on
  y = mean(load_cr_50, 2);
  l = y- mean(load_cr_10, 2);
  u = mean(load_cr_90,2) - y;
  errorbar(n_beacons+INTERVAL, y, l, u, 'o-b','MarkerSize', msz);
  xlabel('number of beacons K');
  ylabel('non-beacon node load');
  ylim([0 1400])
  legend('BVR 2hop', 'S4');
  saveas(gcf, '2hop_nonbeacon_load.eps','psc2')
end
%%%%%%%%figure 1.d beacon node load
for i=1:repeat
  for b=1:length(n_beacons)
    non_beacons = res(b,i).beacons; %we want beacons.. lazy to rename
    load_bvr_10(b,i) = percentile(res(b,i).load_bvr_sf(non_beacons), 10);
    load_bvr_50(b,i) = percentile(res(b,i).load_bvr_sf(non_beacons), 50);
    load_bvr_90(b,i) = percentile(res(b,i).load_bvr_sf(non_beacons), 90);

    load_bvr_2h_10(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 10);
    load_bvr_2h_50(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 50);
    load_bvr_2h_90(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 90);

    load_cr_10(b,i) = percentile(res(b,i).load_cr(non_beacons), 10);
    load_cr_50(b,i) = percentile(res(b,i).load_cr(non_beacons), 50);
    load_cr_90(b,i) = percentile(res(b,i).load_cr(non_beacons), 90);
  end
end
figure(4)
hold on
set(gca, 'FontSize',fontsize);

y = mean(load_bvr_50, 2);
l = y- mean(load_bvr_10, 2);
u = mean(load_bvr_90,2) - y;
errorbar(n_beacons-INTERVAL, y, l, u,'x-r','MarkerSize', msz);

y = mean(load_bvr_2h_50, 2);
l = y- mean(load_bvr_2h_10, 2);
u = mean(load_bvr_2h_90,2) - y;
errorbar(n_beacons+INTERVAL, y, l, u,'v-k','MarkerSize', msz);

y = mean(load_cr_50, 2);
l = y- mean(load_cr_10, 2);
u = mean(load_cr_90,2) - y;
errorbar(n_beacons, y, l, u, 'o-b','MarkerSize', msz);
xlabel('number of beacons K');
ylabel('beacon node load');
xlim([0 105]);
ylim([0 2000]);
legend('BVR', 'BVR 2-hop', 'S4');

saveas(gcf, 'beacon_load.eps','psc2')

%%%%%%%%figure 1.d beacon node load 2hop NEW
for i=1:repeat
  for b=1:length(n_beacons)
    non_beacons = res(b,i).beacons; %we want beacons.. lazy to rename
    load_bvr_10(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 10);
    load_bvr_50(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 50);
    load_bvr_90(b,i) = percentile(res(b,i).load_bvr_2h(non_beacons), 90);

    load_cr_10(b,i) = percentile(res(b,i).load_cr(non_beacons), 10);
    load_cr_50(b,i) = percentile(res(b,i).load_cr(non_beacons), 50);
    load_cr_90(b,i) = percentile(res(b,i).load_cr(non_beacons), 90);
  end
end
figure(14)
set(gca, 'FontSize',fontsize);
y = mean(load_bvr_50, 2);
l = y- mean(load_bvr_10, 2);
u = mean(load_bvr_90,2) - y;
errorbar(n_beacons-INTERVAL, y, l, u,'x-r','MarkerSize', msz);
hold on
y = mean(load_cr_50, 2);
l = y- mean(load_cr_10, 2);
u = mean(load_cr_90,2) - y;
errorbar(n_beacons+INTERVAL, y, l, u, 'o-b','MarkerSize', msz);
xlabel('number of beacons K');
ylabel('beacon node load');
legend('2hop BVR', 'S4');

saveas(gcf, '2hop_beacon_load.eps','psc2')

%%%%%%%%%figure 1.e all node routing state
NEXT_HOP_COST = 2; % 2 bytes to store a nexthop entry
BYTE_PER_S4 = 2+1+NEXT_HOP_COST; %byte per each pcr routing entry
%%bvr routing state calculation
neighbor_count = sum(S.D,2);
%only the n_beacon matters. which nodes are beacons does not matter.
%for b=1:length(n_beacons)
  %bvr_routing_bytes(:,b) = (neighbor_count+1).*n_beacons(b)+ ...
  %    repmat(n_beacons(b)*(NEXT_HOP_COST+2), S.n_nodes, 1);
  %!!!!!!!!change this after v3 is done
%end
%size(bvr_routing_bytes)
%%
for i=1:repeat
  for b=1:length(n_beacons)
    %non_beacons = setdiff(1:S.n_nodes, res(b,i).beacons);
    %non_beacons = 1:S.n_nodes;
%    state_cr_10(b,i) = ...
%	percentile(res(b,i).cluster_cr(non_beacons), 10)*BYTE_PER_S4...
%	+ n_beacons(b)*(NEXT_HOP_COST+2);
    state_cr_10(b,i) = ...
	percentile(routing_state_cr(res(b,i)),0);
%    state_cr_50(b,i) = ...
%	percentile(res(b,i).cluster_cr(non_beacons), 50)*BYTE_PER_S4...
%	+ n_beacons(b)*(NEXT_HOP_COST+2);
    state_cr_50(b,i) = ...
	percentile(routing_state_cr(res(b,i)),50);
%    state_cr_90(b,i) = ...
%	percentile(res(b,i).cluster_cr(non_beacons), 90)*BYTE_PER_S4...
%	+ n_beacons(b)*(NEXT_HOP_COST+2);
    state_cr_90(b,i) = ...
	percentile(routing_state_cr(res(b,i)),100);
    
%    state_bvr_sf_10(b,i) = ...
%	percentile(res(b,i).state_per_node_bvr_sf(non_beacons), ...
%		   10)*n_beacons(b)+n_beacons(b)*(NEXT_HOP_COST+2);
    state_bvr_sf_10(b,i) = ...
	percentile(routing_state_bvr(res(b,i)),0);
%    state_bvr_sf_50(b,i) = ...
%	percentile(res(b,i).state_per_node_bvr_sf(non_beacons), ...
%		   50)*n_beacons(b)+n_beacons(b)*(NEXT_HOP_COST+2);
    state_bvr_sf_50(b,i) = ...
	percentile(routing_state_bvr(res(b,i)),50);
%    state_bvr_sf_90(b,i) = ...
%	percentile(res(b,i).state_per_node_bvr_sf(non_beacons), ...
%		   90)*n_beacons(b)+n_beacons(b)*(NEXT_HOP_COST+2);
    state_bvr_sf_90(b,i) = ...
	percentile(routing_state_bvr(res(b,i)),100);
    
%    state_bvr_2h_10(b,i) = ...
%	percentile(res(b,i).state_per_node_bvr_2h(non_beacons), ...
%		   10)*n_beacons(b)+n_beacons(b)*(NEXT_HOP_COST+2);
    state_bvr_2h_10(b,i) = ...
	percentile(routing_state_bvr(res(b,i),2),0);
%    state_bvr_2h_50(b,i) = ...
%	percentile(res(b,i).state_per_node_bvr_2h(non_beacons), ...
%		   50)*n_beacons(b)+n_beacons(b)*(NEXT_HOP_COST+2);
    state_bvr_2h_50(b,i) = ...
	percentile(routing_state_bvr(res(b,i),2),50);
%    state_bvr_2h_90(b,i) = ...
%	percentile(res(b,i).state_per_node_bvr_2h(non_beacons), ...
%		   90)*n_beacons(b)+n_beacons(b)*(NEXT_HOP_COST+2);
    state_bvr_2h_90(b,i) = ...
	percentile(routing_state_bvr(res(b,i),2),100);
    
  end
end
figure(5)
set(gca, 'FontSize',fontsize);
y = mean(state_cr_50, 2);
l = y - mean(state_cr_10, 2);
u = mean(state_cr_90,2) -y;

errorbar(n_beacons, y, l, u,'o-b','MarkerSize', msz);
hold on

%y = percentile(bvr_routing_bytes(non_beacons, :), 50);
%l = percentile(bvr_routing_bytes(non_beacons, :), 10);
%u = percentile(bvr_routing_bytes(non_beacons, :), 90);
y = mean(state_bvr_sf_50, 2);
l = y - mean(state_bvr_sf_10, 2);
u = mean(state_bvr_sf_90,2) -y;

errorbar(n_beacons-INTERVAL, y, l, u,'x-r','MarkerSize', msz);

y = mean(state_bvr_2h_50, 2);
l = y - mean(state_bvr_2h_10, 2);
u = mean(state_bvr_2h_90,2) -y;

errorbar(n_beacons+INTERVAL, y, l, u,'v-k','MarkerSize', msz);

xlabel('number of beacons K');
xlim([0 105]);
ylabel('routing state per node (Byte)');
legend('S4', 'BVR', 'BVR 2hop',2);
saveas(gcf, 'state.eps','psc2')

%%%%%%%%%figure 1.g compare communication cost
%BROADCAST_PACKET_SIZE=2+1;
%beacon_bd = S.n_nodes*n_beacons*BROADCAST_PACKET_SIZE;
%beacon_bd = beacon_bd';

%DV_ENTRY_SIZE=2+1+1;
for i=1:repeat
  for b=1:length(n_beacons)
    %cr_local_dv_size(b,i) = res(b,i).dv_entries_cr*DV_ENTRY_SIZE;
    [cr_t(b,i), bvr_t(b,i),bvr_2h_t(b,i)] = setup_traffic(res(b,i));
  end
end
figure(7)
set(gca, 'FontSize',fontsize);
%size(mean(cr_local_dv_size, 2))
%size(beacon_bd)
% $$$ y = mean(cr_local_dv_size, 2);
% $$$ l = y - min(cr_local_dv_size,[], 2);
% $$$ u = max(cr_local_dv_size, [], 2) -y;
% $$$ errorbar(n_beacons, y, l, u,'o-b','MarkerSize', msz);
plot_errorbar( cr_t, n_beacons, 'o-b', msz, 1);
tmp = mean(cr_t,2);
fprintf('control overhead of S4, K=56:%.1f\n',tmp(find(n_beacons==56)));
tmp = mean(bvr_t,2);
fprintf('control overhead of BVR 1hop, K=56:%.1f\n',tmp(find(n_beacons==56)));
hold on
%plot(n_beacons, beacon_bd, 'x-r','MarkerSize', msz);
plot_errorbar(bvr_t, n_beacons-INTERVAL, 'x-r',msz,1);
plot_errorbar(bvr_2h_t, n_beacons+INTERVAL, 'v-k',msz,1);
legend('S4', 'BVR', 'BVR 2 hop',2);
xlabel('number of beacons K');
ylabel('control traffic per node (Byte)');
saveas(gcf, 'traffic.eps','psc2')

%%%%%%%%%%%data traffic overhead (main packet header)

if 0
  routing_beacon = 10;
  for b=1:length(n_beacons)
    cr_packet_overhead(b) = 2+1; %beacon ID+distance to beacon
    bvr_packet_overhead(b) = routing_beacon*2;
    %to specify which beacons are routing beacons also incur overhead
    o = factorial(n_beacons(b))/factorial(n_beacons(b)- ...
					  routing_beacon)/factorial(routing_beacon);
    bvr_packet_overhead(b) = bvr_packet_overhead(b) +log(o)/log(2)/8;
  end
  figure(8)
  set(gca, 'FontSize',fontsize);
  hold on
  plot(n_beacons, cr_packet_overhead,'o-b','MarkerSize', msz);
  plot(n_beacons, bvr_packet_overhead,'x-r','MarkerSize', msz);
  plot([0 100],[29 29],'k-');
  legend('S4', 'BVR','default mica2 packet payload size',0);
  xlabel('number of beacons K');
  ylabel('per data packet header overhead (Byte)');
  saveas(gcf, 'header_overhead.eps','psc2')
end